SUBROUTINE massdair_loc( p, masse )
  USE parallel_lmdz
  !
  ! *********************************************************************
  !   ....  Calcule la masse d'air  dans chaque maille   ....
  ! *********************************************************************
  !
  !    Auteurs : P. Le Van , Fr. Hourdin  .
  !   ..........
  !
  !  ..    p                      est  un argum. d'entree pour le s-pg ...
  !  ..  masse                    est un  argum.de sortie pour le s-pg ...
  !
  !  ....  p est defini aux interfaces des llm couches   .....
  !
  IMPLICIT NONE
  !
  include "dimensions.h"
  include "paramet.h"
  include "comgeom.h"
  !
  !  .....   arguments  ....
  !
  REAL :: p(ijb_u:ije_u,llmp1), masse(ijb_u:ije_u,llm)

  !   ....  Variables locales  .....

  INTEGER :: l,ij
  INTEGER :: ijb,ije
  REAL :: massemoyn, massemoys

  REAL :: SSUM
  EXTERNAL SSUM
  !
  !
  !   Methode pour calculer massebx et masseby .
  !   ----------------------------------------
  !
  !    A chaque point scalaire P (i,j) est affecte 4 coefficients d'aires
  !   alpha1(i,j)  calcule  au point ( i+1/4,j-1/4 )
  !   alpha2(i,j)  calcule  au point ( i+1/4,j+1/4 )
  !   alpha3(i,j)  calcule  au point ( i-1/4,j+1/4 )
  !   alpha4(i,j)  calcule  au point ( i-1/4,j-1/4 )
  !
  !    Avec  alpha1(i,j) = aire(i+1/4,j-1/4)/ aire(i,j)
  !
  !    N.B .  Pour plus de details, voir s-pg  ...  iniconst ...
  !
  !
  !
  !   alpha4 .         . alpha1    . alpha4
  !    (i,j)             (i,j)       (i+1,j)
  !
  !         P .        U .          . P
  !       (i,j)       (i,j)         (i+1,j)
  !
  !   alpha3 .         . alpha2    .alpha3
  !    (i,j)              (i,j)     (i+1,j)
  !
  !         V .        Z .          . V
  !       (i,j)
  !
  !   alpha4 .         . alpha1    .alpha4
  !   (i,j+1)            (i,j+1)   (i+1,j+1)
  !
  !         P .        U .          . P
  !      (i,j+1)                    (i+1,j+1)
  !
  !
  !
  !                   On  a :
  !
  !    massebx(i,j) = masse(i  ,j) * ( alpha1(i  ,j) + alpha2(i,j))   +
  !               masse(i+1,j) * ( alpha3(i+1,j) + alpha4(i+1,j) )
  ! localise  au point  ... U (i,j) ...
  !
  !    masseby(i,j) = masse(i,j  ) * ( alpha2(i,j  ) + alpha3(i,j  )  +
  !               masse(i,j+1) * ( alpha1(i,j+1) + alpha4(i,j+1)
  ! localise  au point  ... V (i,j) ...
  !
  !
  !=======================================================================




  ijb=ij_begin-iip1
  ije=ij_end+2*iip1

  IF (pole_nord) ijb=ij_begin
  IF (pole_sud)  ije=ij_end

!$OMP DO SCHEDULE(STATIC,OMP_CHUNK)
  DO   l = 1 , llm
  !
    DO    ij     = ijb, ije
     masse(ij,l) = airesurg(ij) * ( p(ij,l) - p(ij,l+1) )
    ENDDO
  !
    DO   ij = ijb, ije,iip1
     masse(ij+ iim,l) = masse(ij,l)
    ENDDO
  !
  !   DO    ij     = 1,  iim
  !    masse(   ij   ,l) = masse(   ij   ,l) * aire(  ij    )
  !    masse(ij+ip1jm,l) = masse(ij+ip1jm,l) * aire(ij+ip1jm)
  !   ENDDO
  !    massemoyn         = SSUM(iim,masse(   1   ,l),1)/ apoln
  !    massemoys         = SSUM(iim,masse(ip1jm+1,l),1)/ apols
  !   DO    ij     = 1, iip1
  !    masse(   ij   ,l )    = massemoyn
  !    masse(ij+ip1jm,l )    = massemoys
  !   ENDDO

  END DO
!$OMP END DO NOWAIT
  !

END SUBROUTINE massdair_loc
