!$Id: lmdz_physics_distribution.F90 5117 2024-07-24 14:23:34Z abarral $

MODULE lmdz_physics_distribution


CONTAINS

  SUBROUTINE init_physics_distribution(grid_type, nvertex, &
          nbp, nbp_lon, nbp_lat, nbp_lev, &
          communicator)
    USE lmdz_phys_para, ONLY: init_phys_lmdz_para, klon_omp
    USE lmdz_grid_phy, ONLY: init_grid_phy_lmdz
    USE dimphy, ONLY: Init_dimphy
    USE infotrac_phy, ONLY: type_trac
#ifdef REPROBUS
  USE CHEM_REP, ONLY: Init_chem_rep_phys
#endif
    USE lmdz_cppkeys_wrapper, ONLY: CPPKEY_INCA
    IMPLICIT NONE
    INTEGER, INTENT(IN) :: grid_type
    INTEGER, INTENT(IN) :: nvertex
    INTEGER, INTENT(IN) :: nbp
    INTEGER, INTENT(IN) :: nbp_lon
    INTEGER, INTENT(IN) :: nbp_lat
    INTEGER, INTENT(IN) :: nbp_lev
    INTEGER, INTENT(IN) :: communicator

    CALL init_grid_phy_lmdz(grid_type, nvertex, nbp_lon, nbp_lat, nbp_lev)
    CALL init_phys_lmdz_para(nbp, nbp_lon, nbp_lat, nbp_lev, grid_type, nvertex, communicator)
    !$OMP PARALLEL
    CALL init_dimphy(klon_omp, nbp_lev)

    IF (CPPKEY_INCA) THEN
      CALL Init_inca_dim(klon_omp, nbp_lev)
    END IF

#ifdef REPROBUS
! Initialization of Reprobus
    IF (type_trac == 'repr') CALL Init_chem_rep_phys(klon_omp,nbp_lev)
#endif

    !$OMP END PARALLEL

  END SUBROUTINE init_physics_distribution

  !SUBROUTINE Init_Phys_lmdz(iim,jjp1,llm,nb_proc,distrib)
  !  USE lmdz_phys_para, ONLY: Init_phys_lmdz_para!, klon_omp
  !  USE lmdz_grid_phy, ONLY: Init_grid_phy_lmdz!, nbp_lev
  !  USE dimphy, ONLY: Init_dimphy
  !  USE infotrac_phy, ONLY: type_trac
  !#ifdef REPROBUS
  !  USE CHEM_REP, ONLY: Init_chem_rep_phys
  !#endif

  !  IMPLICIT NONE

  !    INTEGER,INTENT(IN) :: iim
  !    INTEGER,INTENT(IN) :: jjp1
  !    INTEGER,INTENT(IN) :: llm
  !    INTEGER,INTENT(IN) :: nb_proc
  !    INTEGER,INTENT(IN) :: distrib(0:nb_proc-1)


  !    CALL Init_grid_phy_lmdz(iim,jjp1,llm)
  !    CALL Init_phys_lmdz_para(iim,jjp1,nb_proc,distrib)
  !!$OMP PARALLEL
  !    CALL Init_dimphy(klon_omp,nbp_lev)

  !#ifdef REPROBUS
  !! Initialization of Reprobus
  !    IF (type_trac == 'repr') CALL Init_chem_rep_phys(klon_omp,nbp_lev)
  !    END IF
  !#endif

  !!$OMP END PARALLEL

  !END SUBROUTINE Init_Phys_lmdz


END MODULE lmdz_physics_distribution

